import { render, screen } from '@testing-library/react'
import { userEvent } from '@testing-library/user-event'
import { FluidType } from 'enums'
import React from 'react'
import { Provider } from 'react-redux'
import UsageEventService from 'services/usageEvent.service'
import {
  createMockEcolyoStore,
  mockChallengeState,
  mockGlobalState,
} from 'tests/__mocks__/store'
import { userChallengeData } from 'tests/__mocks__/userChallengeData.mock'
import ChallengeCardUnlocked from './ChallengeCardUnlocked'

const mockStartUserChallenge = jest.fn()
jest.mock('services/challenge.service', () => {
  return jest.fn(() => ({
    startUserChallenge: mockStartUserChallenge,
  }))
})

jest.mock('services/usageEvent.service')
UsageEventService.addEvent = jest.fn()

describe('ChallengeCardUnlocked component', () => {
  const store = createMockEcolyoStore()
  it('should be rendered correctly', () => {
    render(
      <Provider store={store}>
        <ChallengeCardUnlocked userChallenge={userChallengeData[0]} />
      </Provider>
    )
    expect(screen.getByText('Simone VEILLE')).toBeInTheDocument()
    expect(screen.getByRole('button')).toBeInTheDocument()
    expect(screen.queryAllByRole('dialog').length).toBeFalsy()
  })

  it('should display ChallengeNoFluidModal when launching challenge without configured fluid', async () => {
    render(
      <Provider store={store}>
        <ChallengeCardUnlocked userChallenge={userChallengeData[0]} />
      </Provider>
    )
    await userEvent.click(screen.getByRole('button'))
    expect(screen.queryAllByRole('dialog').length).toBeTruthy()
  })

  it('should not display ChallengeNoFluidModal and update userChallenge when launching challenge with configured fluid', async () => {
    const store = createMockEcolyoStore({
      global: {
        fluidTypes: [FluidType.ELECTRICITY],
        fluidStatus: [{ ...mockGlobalState.fluidStatus[0], status: 200 }],
      },
      challenge: mockChallengeState,
    })
    render(
      <Provider store={store}>
        <ChallengeCardUnlocked userChallenge={userChallengeData[0]} />
      </Provider>
    )
    await userEvent.click(screen.getByRole('button'))
    expect(screen.queryAllByRole('dialog').length).toBeFalsy()
    expect(mockStartUserChallenge).toHaveBeenCalledWith(userChallengeData[0])
  })

  it('should not be able to launch challenge if another one is active', async () => {
    const store = createMockEcolyoStore({
      global: mockGlobalState,
      challenge: {
        ...mockChallengeState,
        currentChallenge: userChallengeData[1],
      },
    })
    render(
      <Provider store={store}>
        <ChallengeCardUnlocked userChallenge={userChallengeData[0]} />
      </Provider>
    )
    const resetButton = screen.getByRole('button')
    expect(resetButton).toHaveAttribute('disabled')
  })
})
