import Button from '@material-ui/core/Button'
import { useClient } from 'cozy-client'
import { OAuthWindow } from 'cozy-harvest-lib/dist/components/OAuthWindow'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { FluidType, UsageEventType } from 'enums'
import { Konnector } from 'models'
import React, { useCallback, useEffect, useState } from 'react'
import UsageEventService from 'services/usageEvent.service'
import { setShouldRefreshConsent } from 'store/global/global.slice'
import { useAppDispatch, useAppSelector } from 'store/hooks'

const GrdfFormOAuth = ({
  onSuccess,
}: {
  onSuccess: (accountId: string) => Promise<void>
}) => {
  const IDLE = 'idle'
  const WAITING = 'waiting'

  const { t } = useI18n()
  const client = useClient()
  const { shouldRefreshConsent, fluidStatus } = useAppSelector(
    state => state.ecolyo.global
  )
  const currentFluidStatus = fluidStatus[FluidType.GAS]
  const konnector: Konnector | null = currentFluidStatus.connection.konnector
  const dispatch = useAppDispatch()
  const [status, setStatus] = useState<string>(IDLE)
  const endOAuth = useCallback(() => {
    setStatus(IDLE)
    // Set back to false the variable that allows to automatically refresh the consent (deletes and recreates the account)
    dispatch(setShouldRefreshConsent(false))
  }, [dispatch])

  const startOAuth = useCallback(async () => {
    // If first connexion, send the usage event
    if (konnector?.slug && currentFluidStatus.lastDataDate === null) {
      await UsageEventService.addEvent(client, {
        type: UsageEventType.KONNECTOR_ATTEMPT_EVENT,
        target: konnector.slug,
        result: 'error',
      })
    }
    setStatus(WAITING)
  }, [client, currentFluidStatus.lastDataDate, konnector])

  const handleAccountId = useCallback(
    (accountId: string) => {
      endOAuth()
      onSuccess(accountId)
    },
    [endOAuth, onSuccess]
  )

  const isWaiting = status === WAITING

  useEffect(() => {
    if (shouldRefreshConsent) {
      // If user has selected accept button on Expired consent modal, his account has been deleted on KonnectorViewerCard, such as his consent (for GRDF), then automatically launch oauth connxion flow
      startOAuth()
    }
  }, [shouldRefreshConsent, startOAuth])

  if (!konnector) {
    return null
  }

  return (
    <>
      <Button
        aria-label={t('auth.accessibility.button_connect')}
        onClick={startOAuth}
        disabled={isWaiting}
        className="btnPrimary"
      >
        {isWaiting
          ? t('auth.connect_oauth.loading')
          : t('auth.button_validate')}
      </Button>
      {isWaiting && (
        <OAuthWindow
          client={client}
          konnector={konnector}
          redirectSlug={client.appMetadata.slug}
          onSuccess={handleAccountId}
          onCancel={endOAuth}
          t={t}
        />
      )}
    </>
  )
}

export default GrdfFormOAuth
