import { render, screen } from '@testing-library/react'
import { userEvent } from '@testing-library/user-event'
import { DataloadState, FluidType } from 'enums'
import { DateTime } from 'luxon'
import { Dataload } from 'models'
import React from 'react'
import { Provider } from 'react-redux'
import { BrowserRouter } from 'react-router-dom'
import UsageEventService from 'services/usageEvent.service'
import {
  baseDataLoad,
  baseMultiFluidDataLoad,
} from 'tests/__mocks__/chartData.mock'
import { createMockEcolyoStore, mockChartState } from 'tests/__mocks__/store'
import DataloadConsumptionVisualizer from './DataloadConsumptionVisualizer'

jest.mock('services/fluidsPrices.service', () => {
  return jest.fn(() => ({
    getAllLastPrices: jest.fn(),
  }))
})

jest.mock(
  'components/ConsumptionVisualizer/DataloadNoValue.tsx',
  () => 'mock-DataloadNoValue'
)
jest.mock(
  'components/ConsumptionVisualizer/EstimatedConsumptionModal.tsx',
  () => 'mock-EstimatedConsumptionModal'
)

const emptyDataLoad = { ...baseDataLoad, value: -1 }
const dataLoadWithValueDetailEmpty: Dataload = {
  ...baseDataLoad,
  valueDetail: [],
}
const dataLoadWithValueDetail: Dataload = {
  ...baseDataLoad,
  valueDetail: [
    { state: DataloadState.VALID, value: 0.612216 },
    { state: DataloadState.VALID, value: 0.675242 },
    { state: DataloadState.VALID, value: 0.6779610000000003 },
  ],
}

describe('Dataload consumption visualizer component', () => {
  const store = createMockEcolyoStore()

  it('should render with single fluid', async () => {
    const { container } = render(
      <Provider store={store}>
        <DataloadConsumptionVisualizer
          fluidType={FluidType.ELECTRICITY}
          dataload={baseDataLoad}
          compareDataload={baseDataLoad}
          setActive={jest.fn()}
        />
      </Provider>
    )
    expect(container).toMatchSnapshot()
  })
  describe('should render nothing', () => {
    it('should render nothing if dataload is null', () => {
      const { container } = render(
        <Provider store={store}>
          <DataloadConsumptionVisualizer
            fluidType={FluidType.ELECTRICITY}
            dataload={null as unknown as Dataload}
            compareDataload={baseDataLoad}
            setActive={jest.fn()}
          />
        </Provider>
      )
      const element = container
        .getElementsByClassName('dataloadvisualizer-root')
        .item(0)
      expect(element?.children.length).toBe(0)
    })
    it('should render nothing if dataload is in the future', () => {
      const { container } = render(
        <Provider store={store}>
          <DataloadConsumptionVisualizer
            fluidType={FluidType.ELECTRICITY}
            dataload={{ ...baseDataLoad, date: DateTime.local(9999, 1, 1) }}
            compareDataload={baseDataLoad}
            setActive={jest.fn()}
          />
        </Provider>
      )
      const element = container
        .getElementsByClassName('dataloadvisualizer-root')
        .item(0)
      expect(element?.children.length).toBe(0)
    })
  })

  it('should render DataloadNoValue if dataload.state is not valid', () => {
    const { container } = render(
      <Provider store={store}>
        <DataloadConsumptionVisualizer
          fluidType={FluidType.ELECTRICITY}
          dataload={{ ...baseDataLoad, state: DataloadState.MISSING }}
          compareDataload={baseDataLoad}
          setActive={jest.fn()}
        />
      </Provider>
    )
    expect(
      container.getElementsByTagName('mock-DataloadNoValue').item(0)
    ).toBeInTheDocument()
  })

  it('should render correctly multifluid and 3 clickable links', () => {
    render(
      <Provider store={store}>
        <BrowserRouter>
          <DataloadConsumptionVisualizer
            fluidType={FluidType.MULTIFLUID}
            dataload={baseMultiFluidDataLoad}
            compareDataload={null}
            setActive={jest.fn()}
          />
        </BrowserRouter>
      </Provider>
    )
    const links = screen.getAllByRole('link')
    expect(links.length).toBe(3)
  })

  it('should render with no value to compare available', async () => {
    const store = createMockEcolyoStore({
      chart: { ...mockChartState, showCompare: true },
    })
    const { container } = render(
      <Provider store={store}>
        <DataloadConsumptionVisualizer
          fluidType={FluidType.WATER}
          dataload={baseDataLoad}
          compareDataload={emptyDataLoad}
          setActive={jest.fn()}
        />
      </Provider>
    )
    const element = container
      .getElementsByClassName('dataloadvisualizer-novalue')
      .item(0)
    expect(element).toBeInTheDocument()
  })
  it('should render with water comparison data', async () => {
    const store = createMockEcolyoStore({
      chart: { ...mockChartState, showCompare: true },
    })
    const { container } = render(
      <Provider store={store}>
        <DataloadConsumptionVisualizer
          fluidType={FluidType.WATER}
          dataload={baseDataLoad}
          compareDataload={baseDataLoad}
          setActive={jest.fn()}
        />
      </Provider>
    )
    const element = container.getElementsByClassName('water-compare').item(0)
    expect(element).toBeInTheDocument()
  })
  it('should render multifluid with no compare and display estimation modal', async () => {
    const { container } = render(
      <Provider store={store}>
        <DataloadConsumptionVisualizer
          fluidType={FluidType.MULTIFLUID}
          dataload={dataLoadWithValueDetailEmpty}
          compareDataload={emptyDataLoad}
          setActive={jest.fn()}
        />
      </Provider>
    )
    const element = container
      .getElementsByTagName('mock-EstimatedConsumptionModal')
      .item(0)
    expect(element).toBeInTheDocument()
  })
  it('should render multifluid with no compare and navigate to singleFluid page', async () => {
    jest.mock('services/converter.service', () => {
      return jest.fn(() => ({
        LoadToEuro: jest.fn(),
      }))
    })

    render(
      <Provider store={store}>
        <BrowserRouter>
          <DataloadConsumptionVisualizer
            fluidType={FluidType.MULTIFLUID}
            dataload={dataLoadWithValueDetail}
            compareDataload={emptyDataLoad}
            setActive={jest.fn()}
          />
        </BrowserRouter>
      </Provider>
    )
    jest.mock('services/usageEvent.service')
    const mockAddEvent = jest.fn()
    UsageEventService.addEvent = mockAddEvent

    const links = screen.getAllByRole('link')
    await userEvent.click(links[0])
    expect(mockAddEvent).toHaveBeenCalled()
  })
})
