import * as Sentry from '@sentry/react'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { FluidSlugType, FluidType, UsageEventType } from 'enums'
import {
  AccountAuthData,
  AccountSgeData,
  FluidConnection,
  UsageEvent,
} from 'models'
import { useState } from 'react'
import AccountService from 'services/account.service'
import ConnectionService from 'services/connection.service'
import UsageEventService from 'services/usageEvent.service'
import { setLoading } from 'store/chart/chart.slice'
import { updateFluidConnection } from 'store/global/global.slice'
import { useAppDispatch, useAppSelector } from 'store/hooks'
import logApp from 'utils/logger'

const useKonnectorAuth = (
  fluidType: FluidType,
  login?: string,
  password?: string
): [() => Promise<null | undefined>, () => Promise<void>, string] => {
  const client = useClient()
  const { t } = useI18n()
  const dispatch = useAppDispatch()
  const { sgeConnect, fluidStatus } = useAppSelector(
    state => state.ecolyo.global
  )
  const currentFluidStatus = fluidStatus[fluidType]
  const konnectorSlug: FluidSlugType =
    currentFluidStatus.connection.konnectorConfig.slug
  const [connectError, setError] = useState<string>('')

  const sendUsageEventError = async (slug: string): Promise<UsageEvent> => {
    return UsageEventService.addEvent(client, {
      type: UsageEventType.KONNECTOR_CONNECT_EVENT,
      target: slug,
      result: 'error',
    })
  }

  const connect = async () => {
    const connectionService = new ConnectionService(client)
    try {
      // If first connexion, send the usage event
      await UsageEventService.addEvent(client, {
        type: UsageEventType.KONNECTOR_ATTEMPT_EVENT,
        target: konnectorSlug,
        result: 'error',
      })
      const { account: _account, trigger: _trigger } =
        konnectorSlug === FluidSlugType.WATER
          ? await connectionService.connectNewUser(konnectorSlug, {
              login: login ? login : '',
              password: password,
            })
          : await connectionService.connectNewUser(
              konnectorSlug,
              undefined,
              sgeConnect
            )
      if (!_trigger || !_account) {
        setError(t('konnector_form.error_account_creation'))
        sendUsageEventError(konnectorSlug)
        return null
      }
      const updatedConnection: FluidConnection = {
        ...currentFluidStatus.connection,
        account: _account,
        trigger: _trigger,
        shouldLaunchKonnector: true,
      }
      setLoading(false)
      dispatch(
        updateFluidConnection({
          fluidType: currentFluidStatus.fluidType,
          fluidConnection: updatedConnection,
        })
      )
    } catch (error) {
      setLoading(false)
      sendUsageEventError(konnectorSlug)
      logApp.error(error)
      Sentry.captureException(error)
    }
  }

  const update = async () => {
    if (currentFluidStatus.connection.account) {
      const _account = currentFluidStatus.connection.account
      let auth: AccountAuthData | AccountSgeData
      if (konnectorSlug === FluidSlugType.WATER) {
        auth = {
          login: login,
          password: password,
        } as AccountAuthData
      } else {
        auth = {
          pointId: sgeConnect.pdl ? sgeConnect.pdl.toString() : '',
          firstname: sgeConnect.firstName,
          lastname: sgeConnect.lastName,
          address: sgeConnect.address,
          postalCode: sgeConnect.zipCode ? sgeConnect.zipCode.toString() : '',
          city: sgeConnect.city,
        } as AccountSgeData
      }
      _account.auth = auth
      const accountService = new AccountService(client)
      const updatedAccount = await accountService.updateAccount(_account)
      const updatedConnection: FluidConnection = {
        ...currentFluidStatus.connection,
        account: updatedAccount,
        shouldLaunchKonnector: true,
      }
      dispatch(
        updateFluidConnection({
          fluidType: currentFluidStatus.fluidType,
          fluidConnection: updatedConnection,
        })
      )
    }
  }
  return [connect, update, connectError]
}

export default useKonnectorAuth
