import { Button, IconButton } from '@material-ui/core'
import Dialog from '@material-ui/core/Dialog'
import * as Sentry from '@sentry/react'
import CloseIcon from 'assets/icons/ico/close.svg'
import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import Icon from 'cozy-ui/transpiled/react/Icon'
import { FluidType, TimeStep } from 'enums'
import FileSaver from 'file-saver'
import { Datachart, Dataload, TimePeriod } from 'models'
import React, { useCallback, useEffect } from 'react'
import ConsumptionDataManager from 'services/consumption.service'
import EnedisMonthlyAnalysisDataService from 'services/enedisMonthlyAnalysisData.service'
import * as XLSX from 'xlsx'
import './exportLoadingModal.scss'

interface ExportDataRow {
  [key: string]: string | number
}

interface ExportDataSheet {
  fluidName: string
  data: ExportDataRow[]
}

interface ExportLoadingModalProps {
  open: boolean
  handleCloseClick: () => void
  handleDone: (e?: unknown) => void
  selectedFluids: FluidType[]
}

const ExportLoadingModal = ({
  open,
  handleCloseClick,
  handleDone,
  selectedFluids,
}: ExportLoadingModalProps) => {
  const { t } = useI18n()
  const client = useClient()

  const fileType =
    'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=UTF-8'
  const fileExtension = '.xlsx'

  const exportToXlsx = (
    exportDataSheets: ExportDataSheet[],
    fileName: string
  ) => {
    const wb = XLSX.utils.book_new()
    for (const dataSheet of exportDataSheets) {
      const ws = XLSX.utils.json_to_sheet(dataSheet.data)
      XLSX.utils.book_append_sheet(wb, ws, dataSheet.fluidName)
    }
    const excelBuffer = XLSX.write(wb, { bookType: 'xlsx', type: 'array' })
    const data = new Blob([excelBuffer], { type: fileType })
    FileSaver.saveAs(data, fileName + fileExtension)
  }

  const buildDataRow = useCallback(
    async (
      dataload: Dataload,
      fluidType: FluidType
    ): Promise<ExportDataRow> => {
      const dataRow: ExportDataRow = {}
      dataRow[t('export.month')] = dataload.date.month
        .toString()
        .padStart(2, '0')
      dataRow[t('export.year')] = dataload.date.year
      dataRow[
        `${t('export.consumption')} (${t(
          `FLUID.${FluidType[fluidType]}.UNIT`
        )})`
      ] = dataload.value
      if (fluidType === FluidType.ELECTRICITY) {
        const emas = new EnedisMonthlyAnalysisDataService(client)
        const maxPowerEntities = await emas.getMaxPowerByDate(
          dataload.date.year,
          dataload.date.month
        )
        if (maxPowerEntities) {
          const maxLoad = maxPowerEntities.reduce((max, entity) => {
            if (entity.load > max) {
              return entity.load
            }
            return max
          }, 0)
          dataRow[t('export.maxpower')] = maxLoad
        }
      }
      return dataRow
    },
    [client, t]
  )

  const getExportDataSheet = useCallback(
    async (fluidType: FluidType): Promise<ExportDataSheet | null> => {
      const consumptionService = new ConsumptionDataManager(client)
      const firstDataDate = await consumptionService.fetchAllFirstDateData(
        [fluidType],
        TimeStep.MONTH
      )
      const lastDataDate = await consumptionService.fetchAllLastDateData(
        [fluidType],
        TimeStep.MONTH
      )
      if (!firstDataDate[0] || !lastDataDate[0]) return null

      const timePeriod: TimePeriod = {
        startDate: firstDataDate[0],
        endDate: lastDataDate[0],
      }

      const dataLoad: Datachart | null = await consumptionService.getGraphData(
        timePeriod,
        TimeStep.MONTH,
        [fluidType],
        undefined,
        undefined,
        false,
        true
      )

      if (!dataLoad?.actualData) return null

      const exportDataSheet: ExportDataSheet = {
        fluidName: t(`FLUID.${FluidType[fluidType]}.LABEL`),
        data: [],
      }

      for (const data of dataLoad.actualData) {
        if (data.value === -1) continue
        const dataRow = await buildDataRow(data, fluidType)
        exportDataSheet.data.push(dataRow)
      }
      return exportDataSheet
    },
    [buildDataRow, client, t]
  )

  useEffect(() => {
    let subscribed = true
    const date = new Date()

    const exportData = async (): Promise<void> => {
      try {
        const exportDataSheets: ExportDataSheet[] = []
        for (const fluidType of selectedFluids) {
          const exportDataFluid = await getExportDataSheet(fluidType)
          if (exportDataFluid) {
            exportDataSheets.push(exportDataFluid)
          }
        }
        await new Promise(r => setTimeout(r, 2000))
        if (subscribed) {
          exportToXlsx(
            exportDataSheets,
            'ecolyo_data_' + date.toLocaleDateString()
          )
          handleDone()
        }
      } catch (e) {
        Sentry.captureException(e)
        handleDone(e)
      }
    }

    if (subscribed && open) {
      exportData()
    }
    return () => {
      subscribed = false
    }
  }, [getExportDataSheet, handleDone, open, selectedFluids])

  return (
    <Dialog
      open={open}
      onClose={(_event, reason) => {
        if (reason !== 'backdropClick' && reason !== 'escapeKeyDown') {
          handleCloseClick()
        }
      }}
      aria-labelledby="accessibility-title"
      classes={{
        root: 'modal-root',
        paper: 'modal-paper',
      }}
    >
      <div id="accessibility-title">
        {t('export.modal_loading.accessibility_title')}
      </div>

      <div className="modal-loading-root">
        <IconButton
          aria-label={t('export.button_close')}
          className="modal-paper-close-button"
          onClick={handleCloseClick}
        >
          <Icon icon={CloseIcon} size={18} />
        </IconButton>
        <div className="content">
          <div className="icon-main">
            <Loader color="gold" />
          </div>
          <div className="text-16-bold subtitle">
            {t('export.modal_loading.text1')}
          </div>
          <div>{t('export.modal_loading.text2')}</div>
          <Button
            aria-label={t('export.modal_loading.button_cancel')}
            onClick={handleCloseClick}
            className="btnSecondary"
          >
            {t('export.modal_loading.button_cancel')}
          </Button>
        </div>
      </div>
    </Dialog>
  )
}

export default ExportLoadingModal
